local addonName, at = ...

if select(2, UnitClass("player")) ~= 'HUNTER' then
	DisableAddOn(addonName)
	return
 end

local core = CreateFrame('Frame', nil, UIParent)
core:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
at.core = core

local callbacks = {}
function at.Callback(name, ...)
	local arg1, arg2 = ...
	local tab = callbacks[name] or {}
	if arg1 and not arg2 and type(arg1) == 'function' then
		tinsert(tab, arg1)
		callbacks[name] = tab
	else
		--print(name)
		for x =1, #tab do
			tab[x](...)
		end
	end
end


-- both use this so I'll just keep these here.
at.NewTicker = C_Timer.NewTicker
at.GetTime = GetTime


local units = {}
local CallSpells = { 883, 83242, 83243, 83244, 83245 } -- Call Pet 1-5
at.pets = {}
at.spells = {}


local Pet_Called = false
local Pet_Dismissed = false
at.mediaPath = "Interface\\AddOns\\" .. addonName .. "\\Media\\"





-- id, name, description, icon, background, role = GetSpecializationInfo(specIndex [, isInspect [, isPet]])
-- maybe add spec icon to the portrait in the CharacterFrame pet tab, it already shows the players


at.spells[883] = function()
	Pet_Called = true
end
at.spells[83242] = at.spells[883]
at.spells[83243] = at.spells[883]
at.spells[83244] = at.spells[883]
at.spells[83245] = at.spells[883]

at.spells[2641] = function()
	Pet_Dismissed = true
end

local function UpdatePet()
	if PetHasActionBar() and UnitIsVisible("pet") and not at.slot then
		-- get the currently active Call Pet #, this doubles as a stable slotID
		for x =1, #CallSpells do
			if IsCurrentSpell(CallSpells[x]) then
				at.slot = x
				break
			end
		end
		
		--local id  = tonumber(UnitGUID('pet'):match("-(%d+)-%x+$"), 10)
		local icon, name, level, family, talent = GetStablePetInfo( at.slot )
		
		local tab = at.pets[ at.slot ] -- not sure what to do with the extra data yet, but I'm sure I'll think of something
		tab.id = tonumber(UnitGUID('pet'):match("-(%d+)-%x+$"), 10) -- ModelFrame:SetCreature( id )
		tab.name = name
		tab.level = level
		tab.family = family
		
		tab.icon = icon			-- why not?
		tab.talent = talent
		tab.specialization = GetSpecialization(nil, true)
		tab.food = { GetStablePetFoodTypes( at.slot ) }
		
		if Pet_Called then
			tab.called = tab.called +1
			Pet_Called = false
			at.Callback('OnPetCalled')
			
		else
			at.Callback('OnPetAppeared')
			
		end
		
		at.pets[ at.slot ] = tab
		
		core:RegisterUnitEvent("UNIT_NAME_UPDATE", 'pet')
		
	elseif at.slot then
		local slot = at.slot
		if Pet_Dismissed then
			at.slot = nil
			at.Callback('OnPetDismissed', slot)
			Pet_Dismissed = false
		
		elseif not UnitIsVisible("pet") then
			at.slot = nil
			at.Callback('OnPetVanished', slot)
		
		end
		
		core:UnregisterEvent("UNIT_NAME_UPDATE")
		
	end
end

core:RegisterEvent("UNIT_HEALTH")
function core:UNIT_HEALTH(token)
	if token == 'pet' and at.slot then
		at.Callback('OnPetHealthChanged')
	elseif UnitIsTappedByPlayer(token) or UnitIsTappedByAllThreatList(token) then
		local guid = UnitGUID(token)
		if not units[ guid ] and not UnitIsTrivial(token) then
			units[ guid ] = UnitLevel(token)
		end
	end
end
core:RegisterEvent("PLAYER_REGEN_ENABLED")
function core:PLAYER_REGEN_ENABLED(token)
	units = wipe( units )
end


function core:UNIT_NAME_UPDATE(token)
	UpdatePet()
	at.pets[ at.slot ].name = UnitName('pet')
end

core:RegisterEvent('PLAYER_LEVEL_UP')
--function core:UNIT_LEVEL(level)
function core:PLAYER_LEVEL_UP(level)
	if at.slot then
		at.pets[ at.slot ].level = level --UnitLevel('pet')
	end
	at.Callback('OnPlayerLevelUp', level)
end



core.VariableLoadCount = 0
local function LoadVariables(self, addon)
	-- this will run for each event that loads some form of variable; only doing something once they've all loaded.
	if addon then
		if addon == addonName then
			self.VariableLoadCount = self.VariableLoadCount +1
			self:UnregisterEvent('ADDON_LOADED')
		end
	else
		self.VariableLoadCount = self.VariableLoadCount +1
	end
	if self.VariableLoadCount > 2 then
		PetshipSetting = PetshipSetting or {}
		PetshipDB = PetshipDB or {}
		at.pets = PetshipDB.pets or {}
		
		at.playerID = UnitGUID('player')
		
		for x = 1, 55 do
			if not at.pets[x] or not at.pets[x].id then
				local icon, name, level, family, talent = GetStablePetInfo( x )
				if icon and name and level and family then
					at.pets[x] = {
						name = name,
						family = family,
						level = level,
						icon = icon, -- not needed but hey, whatever.
						talent = talent, -- ??
						
						called = 0,
						died = 0,
					}
					at.Callback('OnStableInfoAdded', x, icon, name, level, family, talent)
				else
					at.pets[x] = {
						called = 0,
						died = 0,
					}
					at.Callback('OnStableInfoAdded', x)
				end
			end
		end
		
		at.Callback('AddonLoaded')
		
		self.VariableLoadCount = nil
		self:UnregisterEvent('VARIABLES_LOADED')
		self:UnregisterEvent('PLAYER_ENTERING_WORLD')
	end
end

core:RegisterEvent('PLAYER_ENTERING_WORLD')
core.PLAYER_ENTERING_WORLD = LoadVariables

core:RegisterEvent('ADDON_LOADED')
core.ADDON_LOADED = LoadVariables

core:RegisterEvent('VARIABLES_LOADED')
core.VARIABLES_LOADED = LoadVariables

core:RegisterEvent('PLAYER_LOGOUT')
function core:PLAYER_LOGOUT()
	at.Callback('Logout')
	PetshipDB = at.pets
end


-- hooks
hooksecurefunc('SetPetSlot', function(oldSlot, newSlot)
	local tab = at.pets[oldSlot]
	at.pets[newSlot] = at.pets[oldSlot]
	at.pets[oldSlot] = tab
	
	if oldSlot == at.slot or newSlot == at.slot then -- "Did you just put that animal in your inventory?" - Fidget
		PetDismissed = true
		UpdatePet()
	end
end)
hooksecurefunc('PetAbandon', function()
	local tab = wipe( at.pets[ at.slot ] ) -- it will remove a table inside itself but how often will this happen...
	tab.called = 0
	tab.died = 0
	
	at.pets[ at.slot ] = tab
	at.Callback('OnPetAbandoned')
end)
hooksecurefunc('LockPetActionBar', UpdatePet)
hooksecurefunc('UnlockPetActionBar', UpdatePet)
hooksecurefunc('HidePetActionBar', UpdatePet)
hooksecurefunc('ShowPetActionBar', UpdatePet)



core:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
function core:COMBAT_LOG_EVENT_UNFILTERED(...)
	--local timestamp, event, hideCaster, srcGUID, srcName, srcFlags, srcFlags2, destGUID, destName, destFlags, destFlags2, spellID, spellName, spellFlag, auraType, count = ...
	local _, event, _, srcGUID, _, _, _, destGUID, _, _, _, spellID, _, _, _, _ = ...

	if event == 'SPELL_CAST_SUCCESS' and srcGUID == at.playerID then
		local sub = at.spells[spellID]
		if sub then
			sub() -- individualized callback functions, since any singular part will use only 1 spellID.
		end
		
	elseif event == 'UNIT_DIED' then
		if UnitGUID('pet') == destGUID then
			at.Callback('PetDied')
			
		elseif units[destGUID] then
			at.Callback('PlayerKilledUnit', units[destGUID])
			units[destGUID] = nil
		end

	end
end





_G['SLASH_' .. addonName .. 1] = "/petship"
SlashCmdList[addonName] = function(msg)
	if InCombatLockdown() then return print('Commands disabled in combat.') end
	if msg == 'reset' then
		PetshipDB = nil
		core:UnregisterEvent('PLAYER_LOGOUT')
		ReloadUI()
	end
end



