local addonName, at = ...
if not at.core then return end

local core = at.core

local NewTicker = at.NewTicker

local Loyalty = { 0, 30, 45, 60, 90 }
local Ranks = {
	[0] = 'Rebellious',
	[1] = 'Unruly',
	[2] = 'Submissive',
	[3] = 'Dependable',
	[4] = 'Faithful',
	[5] = 'Best Friend',
}
local Scale = 1
local KillsToLevel = 100
local isMaxLevel = false

local PetTimer
local PetTimer_LastTick = 0
local uptime = 0

local XP_Previous = 0


-- frames/textures
local LoyaltyFrame = CreateFrame('Frame', nil, PetModelFrame)
LoyaltyFrame:SetSize(100, 16)

local LoyaltyFontString = LoyaltyFrame:CreateFontString()
LoyaltyFontString:SetFont([[Fonts\FRIZQT__.TTF]], 14, 'NONE')
LoyaltyFontString:SetTextColor(1.0, 1.0, 0.0)
LoyaltyFontString:SetPoint('TOP', PetModelFrame, 'TOP')
LoyaltyFrame:SetAllPoints(LoyaltyFontString)
LoyaltyFontString:SetJustifyH('CENTER')
LoyaltyFontString:SetText( Ranks[ 0 ] )

-- maybe display images to determine loyalty level; possible icons: Achievement_Level_# 10, 20, 30... 85... 90, 100


local function UpdateLoyalty()
	if at.slot then
		local tab = at.pets[ at.slot ]
		--[[if tab.happiness < 350 then -- TODO
			tab.xp = floor(tab.xp * 0.93)
		end--]]
		
		if isMaxLevel then
			if tab.xp >= KillsToLevel * ( tab.loyalty +1 ) then
				tab.loyalty = tab.loyalty +1
			end
			
		elseif tab.xp >= UnitXPMax('player') * 0.05 and tab.active > Loyalty[ tab.loyalty +1 ] * Scale then
			-- yes, if the player levels up it will increase the required xp
			tab.xp = 0
			tab.loyalty = tab.loyalty +1
			
		end
		
		LoyaltyFontString:SetText( Ranks[ tab.loyalty ] )
		
		at.pets[ at.slot ] = tab
	end
end

local function PetTimer_OnTick()
	local dub = GetTime()
	uptime = uptime + floor((dub - PetTimer_LastTick) / 60) -- get the difference and add it
	PetTimer_LastTick = dub
	
	
	--[[local tab = at.pets[ at.slot ]
	if tab.happiness < 350 then -- TODO
		tab.xp = floor(tab.xp * 0.93)
	end--]]
	UpdateLoyalty()
end
local function PetTimer_Stop(slot)
	if PetTimer then
		if slot then
			PetTimer_OnTick() -- do one last tick
			at.pets[ slot ].active = at.pets[ slot ].active + (uptime * Scale)
			uptime = at.pets[ slot ].active
		end
		
		PetTimer:Cancel()
		PetTimer = nil
	end
end
local function PetActive()
	--if not PetTimer and at.slot then
		PetTimer = NewTicker(60, PetTimer_OnTick) -- infinite iteration; check pet uptime every minute
		PetTimer_LastTick = GetTime()
		uptime = at.pets[ at.slot ].active
		
		UpdateLoyalty()
	--end
end

local function Register_PlayerKill()
	if at.pets[ at.slot ].loyalty < #Ranks -1 then
		at.pets[ at.slot ].xp = 0 -- now we'll just track kills
		at.Callback('PlayerKilledUnit', function(level)
			-- basically it'll be based off how many kills you get
			if at.slot then
				at.pets[ at.slot ].xp = at.pets[ at.slot ].xp + 1
				UpdateLoyalty()
			end
		end)
	end
end
at.Callback('AddonLoaded', function()
	XP_Previous = UnitXP('player')
	Scale = PetshipSetting.LoyaltyScale or Scale
	KillsToLevel = PetshipSetting.KillsToLevel or KillsToLevel
	if UnitLevel('player') == GetMaxPlayerLevel() then
		isMaxLevel = true
		Register_PlayerKill()
	else
		core:RegisterEvent('PLAYER_XP_UPDATE')
		core:RegisterEvent('QUEST_TURNED_IN')
		function core:PLAYER_XP_UPDATE(token)
			if at.slot then
				at.pets[ at.slot ] = at.pets[ at.slot ] + UnitXP(token)
			end
		end
		function core:QUEST_TURNED_IN()
			self:RegisterEvent('QUEST_LOG_UPDATE')
			self:UnregisterEvent('PLAYER_XP_UPDATE')
		end
		function core:QUEST_LOG_UPDATE()
			self:UnregisterEvent('QUEST_LOG_UPDATE')
			self:RegisterEvent('PLAYER_XP_UPDATE')
		end
	end
end)
at.Callback('Logout', function()
	if PetTimer then PetTimer_Stop() end
	PetshipSetting.LoyaltyScale = Scale
	PetshipSetting.KillsToLevel = KillsToLevel
end)

at.Callback('OnPlayerLevelUp', function(level)
	if level == GetMaxPlayerLevel() then
		isMaxLevel = true
		Register_PlayerKill()
		core:UnregisterEvent('QUEST_LOG_UPDATE')
		core:UnregisterEvent('PLAYER_XP_UPDATE')
		core:UnregisterEvent('QUEST_TURNED_IN')
	end
end)

at.Callback('OnPetDismissed', PetTimer_Stop)
at.Callback('OnPetVanished', PetTimer_Stop)
at.Callback('OnPetAbandoned', PetTimer_Stop)

at.Callback('OnPetCalled', PetActive)
at.Callback('OnPetAppeared', PetActive)

--at.Callback('OnPetUpdate', UpdateLoyalty)

at.Callback('OnStableInfoAdded', function(slot, icon, name, level, family, talent)
	at.pets[ slot ].loyalty = at.pets[ slot ].loyalty or 0
	at.pets[ slot ].active = at.pets[ slot ].active or 0
	at.pets[ slot ].xp = at.pets[ slot ].xp or 0
end)






